# Auto-generated file, do not edit
from Keywords import Keyword as Kw
Legacy = Kw("Legacy")
Core = Kw("Core")
Development = Kw("Development")
Building = Kw("Building")
Debugger = Kw("Debugger")
IDE = Kw("IDE")
GUIDesigner = Kw("GUIDesigner")
Profiling = Kw("Profiling")
RevisionControl = Kw("RevisionControl")
Translation = Kw("Translation")
Office = Kw("Office")
Calendar = Kw("Calendar")
ContactManagement = Kw("ContactManagement")
Database = Kw("Database")
Dictionary = Kw("Dictionary")
Chart = Kw("Chart")
Email = Kw("Email")
Finance = Kw("Finance")
FlowChart = Kw("FlowChart")
PDA = Kw("PDA")
ProjectManagement = Kw("ProjectManagement")
Presentation = Kw("Presentation")
Spreadsheet = Kw("Spreadsheet")
WordProcessor = Kw("WordProcessor")
Graphics = Kw("Graphics")
x2DGraphics = Kw("2DGraphics")
VectorGraphics = Kw("VectorGraphics")
RasterGraphics = Kw("RasterGraphics")
x3DGraphics = Kw("3DGraphics")
Scanning = Kw("Scanning")
OCR = Kw("OCR")
Photograph = Kw("Photograph")
Viewer = Kw("Viewer")
Settings = Kw("Settings")
DesktopSettings = Kw("DesktopSettings")
HardwareSettings = Kw("HardwareSettings")
PackageManager = Kw("PackageManager")
Network = Kw("Network")
Dialup = Kw("Dialup")
InstantMessaging = Kw("InstantMessaging")
IRCClient = Kw("IRCClient")
FileTransfer = Kw("FileTransfer")
HamRadio = Kw("HamRadio")
News = Kw("News")
P2P = Kw("P2P")
RemoteAccess = Kw("RemoteAccess")
Telephony = Kw("Telephony")
WebBrowser = Kw("WebBrowser")
WebDevelopment = Kw("WebDevelopment")
AudioVideo = Kw("AudioVideo")
Audio = Kw("Audio")
Midi = Kw("Midi")
Mixer = Kw("Mixer")
Sequencer = Kw("Sequencer")
Tuner = Kw("Tuner")
Video = Kw("Video")
TV = Kw("TV")
AudioVideoEditing = Kw("AudioVideoEditing")
Player = Kw("Player")
Recorder = Kw("Recorder")
DiscBurning = Kw("DiscBurning")
Game = Kw("Game")
ActionGame = Kw("ActionGame")
AdventureGame = Kw("AdventureGame")
ArcadeGame = Kw("ArcadeGame")
BoardGame = Kw("BoardGame")
BlocksGame = Kw("BlocksGame")
CardGame = Kw("CardGame")
KidsGame = Kw("KidsGame")
LogicGame = Kw("LogicGame")
RolePlaying = Kw("RolePlaying")
Simulation = Kw("Simulation")
SportsGame = Kw("SportsGame")
StrategyGame = Kw("StrategyGame")
Education = Kw("Education")
Art = Kw("Art")
Construction = Kw("Construction")
Music = Kw("Music")
Languages = Kw("Languages")
Science = Kw("Science")
Astronomy = Kw("Astronomy")
Biology = Kw("Biology")
Chemistry = Kw("Chemistry")
Geology = Kw("Geology")
Math = Kw("Math")
MedicalSoftware = Kw("MedicalSoftware")
Physics = Kw("Physics")
Teaching = Kw("Teaching")
Amusement = Kw("Amusement")
Applet = Kw("Applet")
Archiving = Kw("Archiving")
Electronics = Kw("Electronics")
Emulator = Kw("Emulator")
Engineering = Kw("Engineering")
FileManager = Kw("FileManager")
Shell = Kw("Shell")
Screensaver = Kw("Screensaver")
TerminalEmulator = Kw("TerminalEmulator")
TrayIcon = Kw("TrayIcon")
System = Kw("System")
Filesystem = Kw("Filesystem")
Monitor = Kw("Monitor")
Security = Kw("Security")
Utility = Kw("Utility")
Accessibility = Kw("Accessibility")
Calculator = Kw("Calculator")
Clock = Kw("Clock")
TextEditor = Kw("TextEditor")
Qt = Kw("Qt")
GTK = Kw("GTK")
KDE = Kw("KDE")
GNOME = Kw("GNOME")
Motif = Kw("Motif")
Java = Kw("Java")
ConsoleOnly = Kw("ConsoleOnly")
